@@ -742,7 +742,14 @@ char *mlt_animation_serialize_cut_tf(mlt_animation self,
             while (used + item_len + 2 > size) // +2 for ';' and NULL
             {
                 size += 1000;
-                ret = realloc(ret, size);
+                char *tmp = realloc(ret, size);
+                if (!tmp) {
+                    free(ret);
+                    mlt_property_close(item.property);
+                    mlt_property_close(time_property);
+                    return NULL;
+                }
+                ret = tmp;
             }
 
             // Append item delimiter (;) if needed.
@@ -1101,11 +1108,11 @@ static inline double catmull_rom_interpolate(double x0,
     double t12 = pow(distance(x1, y1, x2, y2), alpha);
     if (tension > 0.0 || (y1 < y0 && y1 > y2) || (y1 > y0 && y1 < y2)) {
         double t01 = pow(distance(x0, y0, x1, y1), alpha);
-        m1 = abs(tension) * (y2 - y1 + t12 * ((y1 - y0) / t01 - (y2 - y0) / (t01 + t12)));
+        m1 = fabs(tension) * (y2 - y1 + t12 * ((y1 - y0) / t01 - (y2 - y0) / (t01 + t12)));
     }
     if (tension > 0.0 || (y2 < y1 && y2 > y3) || (y2 > y1 && y2 < y3)) {
         double t23 = pow(distance(x2, y2, x3, y3), alpha);
-        m2 = abs(tension) * (y2 - y1 + t12 * ((y3 - y2) / t23 - (y3 - y1) / (t12 + t23)));
+        m2 = fabs(tension) * (y2 - y1 + t12 * ((y3 - y2) / t23 - (y3 - y1) / (t12 + t23)));
     }
     double a = 2.0 * (y1 - y2) + m1 + m2;
     double b = -3.0 * (y1 - y2) - m1 - m1 - m2;
@@ -1347,7 +1354,7 @@ static inline double interpolate_value(double x0,
  * \private \memberof mlt_animation_s
  *
  * \param item an unpopulated animation item to be interpolated.
- *  The frame and keyframe_type fields must already be set. The value for "frame" is the postion
+ *  The frame and keyframe_type fields must already be set. The value for "frame" is the position
  *  at which the value will be interpolated. The value for "keyframe_type" determines which
  *  interpolation will be used.
  * \param p a sequential array of 4 animation items. The frame value for item must lie between the
@@ -1513,4 +1520,4 @@ static int interpolate_item(mlt_animation_item item,
         mlt_property_pass(item->property, p[1]->property);
     }
     return error;
-}
\ No newline at end of file
+}
