@@ -23,12 +23,13 @@
 #ifndef MLT_POOL_H
 #define MLT_POOL_H
 
-extern void mlt_pool_init();
-extern void *mlt_pool_alloc(int size);
-extern void *mlt_pool_realloc(void *ptr, int size);
-extern void mlt_pool_release(void *release);
-extern void mlt_pool_purge();
-extern void mlt_pool_close();
-extern void mlt_pool_stat();
+#include "mlt_export.h"
+MLT_EXPORT void mlt_pool_init();
+MLT_EXPORT void *mlt_pool_alloc(int size);
+MLT_EXPORT void *mlt_pool_realloc(void *ptr, int size);
+MLT_EXPORT void mlt_pool_release(void *release);
+MLT_EXPORT void mlt_pool_purge();
+MLT_EXPORT void mlt_pool_close();
+MLT_EXPORT void mlt_pool_stat();
 
 #endif
