@@ -1,6 +1,6 @@
 /*
  * melt.c -- MLT command line utility
- * Copyright (C) 2002-2024 Meltytech, LLC
+ * Copyright (C) 2002-2025 Meltytech, LLC
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 #define _GNU_SOURCE
 #endif
 #include <libgen.h>
-#include <limits.h>
 #include <locale.h>
 #include <sched.h>
 #include <signal.h>
@@ -33,6 +32,7 @@
 #include <framework/mlt.h>
 
 #if (defined(__APPLE__) || defined(_WIN32) || defined(HAVE_SDL2)) && !defined(MELT_NOSDL)
+#define SDL_MAIN_HANDLED
 #include <SDL.h>
 #endif
 
@@ -339,7 +339,7 @@ static int load_consumer(mlt_consumer *consumer, mlt_profile profile, int argc,
                     mlt_properties_parse(new_props, argv[++i]);
             }
         }
-    } else
+    } else {
         for (i = 1; i < argc; i++) {
             if (!strcmp(argv[i], "-consumer")) {
                 if (*consumer)
@@ -352,6 +352,7 @@ static int load_consumer(mlt_consumer *consumer, mlt_profile profile, int argc,
                 }
             }
         }
+    }
     return EXIT_SUCCESS;
 }
 
@@ -514,6 +515,8 @@ static void show_usage(char *program_name)
         "  -null-track | -hide-track                Add a hidden track\n"
         "  -profile name                            Set the processing settings\n"
         "  -progress                                Display progress along with position\n"
+        "  -progress2                               Display progress along with position on a new "
+        "line\n"
         "  -query                                   List all of the registered services\n"
         "  -query \"consumers\" | \"consumer\"=id       List consumers or show info about one\n"
         "  -query \"filters\" | \"filter\"=id           List filters or show info about one\n"
@@ -945,6 +948,10 @@ int main(int argc, char **argv)
 
     // Look for the consumer option to load profile settings from consumer properties
     backup_profile = mlt_profile_clone(profile);
+
+    // Try to initialize QApplication on the main thread to prevent crash
+    mlt_filter_close(mlt_factory_filter(profile, "qtcrop", NULL));
+
     if (load_consumer(&consumer, profile, argc, argv) != EXIT_SUCCESS)
         goto exit_factory;
 
@@ -1103,10 +1110,11 @@ int main(int argc, char **argv)
         }
     } else {
         show_usage(argv[0]);
+        error = EXIT_FAILURE;
     }
 
     // Disconnect producer from consumer to prevent ref cycles from closing services
-    if (consumer) {
+    if (consumer && !error) {
         error = mlt_properties_get_int(MLT_CONSUMER_PROPERTIES(consumer), "melt_error");
         mlt_consumer_connect(consumer, NULL);
         if (!is_abort)
