@@ -1,6 +1,6 @@
 /*
  * filter_avcolour_space.c -- Colour space filter
- * Copyright (C) 2004-2024 Meltytech, LLC
+ * Copyright (C) 2004-2025 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -71,6 +71,9 @@ static int convert_mlt_to_av_cs(mlt_image_format format)
     case mlt_image_yuv444p10:
         value = AV_PIX_FMT_YUV444P10LE;
         break;
+    case mlt_image_rgba64:
+        value = AV_PIX_FMT_RGBA64LE;
+        break;
     default:
         mlt_log_error(NULL,
                       "[filter avcolor_space] Invalid format %s\n",
@@ -90,8 +93,8 @@ static int av_convert_image(uint8_t *out,
                             int out_height,
                             int in_width,
                             int in_height,
-                            int src_colorspace,
-                            int dst_colorspace,
+                            mlt_colorspace src_colorspace,
+                            mlt_colorspace dst_colorspace,
                             int src_full_range,
                             int dst_full_range)
 {
@@ -115,7 +118,7 @@ static int av_convert_image(uint8_t *out,
     if (context) {
         // libswscale wants the RGB colorspace to be SWS_CS_DEFAULT, which is = SWS_CS_ITU601.
         if (out_fmt == AV_PIX_FMT_RGB24 || out_fmt == AV_PIX_FMT_RGBA)
-            dst_colorspace = 601;
+            dst_colorspace = mlt_colorspace_bt601;
         error = mlt_set_luma_transfer(context,
                                       src_colorspace,
                                       dst_colorspace,
@@ -152,8 +155,9 @@ static int convert_image(mlt_frame frame,
     if (*format != output_format || out_width) {
         mlt_profile profile = mlt_service_profile(
             MLT_PRODUCER_SERVICE(mlt_frame_get_original_producer(frame)));
-        int profile_colorspace = profile ? profile->colorspace : 601;
-        int colorspace = mlt_properties_get_int(properties, "colorspace");
+        mlt_colorspace profile_colorspace = profile ? profile->colorspace : mlt_colorspace_bt601;
+        const char *colorspace_str = mlt_properties_get(properties, "colorspace");
+        mlt_colorspace colorspace = mlt_image_colorspace_id(colorspace_str);
         int width = mlt_properties_get_int(properties, "width");
         int height = mlt_properties_get_int(properties, "height");
         int src_full_range = mlt_properties_get_int(properties, "full_range");
@@ -187,7 +191,7 @@ static int convert_image(mlt_frame frame,
         uint8_t *output = mlt_pool_alloc(size);
 
         if (out_width == width && out_height == height) {
-            if (*format == mlt_image_rgba) {
+            if (*format == mlt_image_rgba && output_format != mlt_image_rgba64) {
                 register int len = width * height;
                 uint8_t *alpha = mlt_pool_alloc(len);
 
@@ -227,6 +231,20 @@ static int convert_image(mlt_frame frame,
                     }
                     mlt_frame_set_alpha(frame, alpha, len, mlt_pool_release);
                 }
+            } else if (*format == mlt_image_rgba64 && output_format != mlt_image_rgba) {
+                // Extract the alpha mask from the RGBA image
+                int len = width * height;
+                uint8_t *alpha = mlt_pool_alloc(len);
+                if (alpha) {
+                    const uint16_t *s = ((uint16_t *) *image) + 3; // start on the alpha component
+                    uint8_t *d = alpha;
+                    for (int i = 0; i < len; i++) {
+                        *d = lrint((float) *s / 256.0);
+                        d++;
+                        s += 4;
+                    }
+                    mlt_frame_set_alpha(frame, alpha, len, mlt_pool_release);
+                }
             }
         } else {
             // Scaling
@@ -253,12 +271,11 @@ static int convert_image(mlt_frame frame,
                 mlt_properties_set_int(properties, "colorspace", profile_colorspace);
             mlt_properties_set_int(properties, "full_range", dst_full_range);
         }
-        *image = output;
-        *format = output_format;
         mlt_frame_set_image(frame, output, size, mlt_pool_release);
+        *image = output;
 
-        if (out_width == width && out_height == height)
-            if (output_format == mlt_image_rgba) {
+        if (out_width == width && out_height == height) {
+            if (output_format == mlt_image_rgba && *format != mlt_image_rgba64) {
                 register int len = width * height;
                 int alpha_size = 0;
                 uint8_t *alpha = mlt_frame_get_alpha_size(frame, &alpha_size);
@@ -298,8 +315,23 @@ static int convert_image(mlt_frame frame,
                         } while (--n > 0);
                     }
                 }
+            } else if (output_format == mlt_image_rgba64 && *format != mlt_image_rgba) {
+                int len = width * height;
+                int alpha_size = 0;
+                uint8_t *alpha = mlt_frame_get_alpha_size(frame, &alpha_size);
+                if (alpha && alpha_size >= len) {
+                    const uint8_t *s = alpha;
+                    uint16_t *d = ((uint16_t *) *image) + 3; // start on the alpha component
+                    for (int i = 0; i < len; i++) {
+                        *d = *s << 8;
+                        d += 4;
+                        s++;
+                    }
+                }
             }
+        }
 
+        *format = output_format;
         mlt_properties_set_int(properties, "format", output_format);
         mlt_properties_set_int(properties, "width", out_width);
         mlt_properties_set_int(properties, "height", out_height);
