@@ -1,6 +1,6 @@
 /*
  * link_avfilter.c -- provide various links based on libavfilter
- * Copyright (C) 2023-2024 Meltytech, LLC
+ * Copyright (C) 2023-2025 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -54,6 +54,8 @@ typedef struct
     int format;
     int width;
     int height;
+    mlt_colorspace colorspace;
+    int full_range;
     int channels;
     int frequency;
     int reset;
@@ -388,8 +390,13 @@ fail:
     avfilter_graph_free(&pdata->avfilter_graph);
 }
 
-static void init_image_filtergraph(
-    mlt_link self, mlt_image_format format, int width, int height, double resolution_scale)
+static void init_image_filtergraph(mlt_link self,
+                                   mlt_image_format format,
+                                   int width,
+                                   int height,
+                                   mlt_colorspace colorspace,
+                                   int full_range,
+                                   double resolution_scale)
 {
     private_data *pdata = (private_data *) self->child;
     mlt_profile profile = mlt_service_profile(MLT_LINK_SERVICE(self));
@@ -402,11 +409,15 @@ static void init_image_filtergraph(
     AVRational sar = (AVRational){profile->sample_aspect_num, profile->sample_aspect_den};
     AVRational timebase = (AVRational){profile->frame_rate_den, profile->frame_rate_num};
     AVRational framerate = (AVRational){profile->frame_rate_num, profile->frame_rate_den};
+    int avcolorspace = mlt_to_av_colorspace(colorspace, height);
+    int color_range = mlt_to_av_color_range(full_range);
     int ret;
 
     pdata->format = format;
     pdata->width = width;
     pdata->height = height;
+    pdata->colorspace = colorspace;
+    pdata->full_range = full_range;
 
     // Set up formats
     pixel_fmts[0] = mlt_to_av_image_format(format);
@@ -469,6 +480,16 @@ static void init_image_filtergraph(
         mlt_log_error(self, "Cannot set src frame_rate %d/%d\n", framerate.num, framerate.den);
         goto fail;
     }
+    ret = av_opt_set_int(pdata->avbuffsrc_ctx, "colorspace", avcolorspace, AV_OPT_SEARCH_CHILDREN);
+    if (ret < 0) {
+        mlt_log_error(self, "Cannot set src colorspace %d\n", avcolorspace);
+        goto fail;
+    }
+    ret = av_opt_set_int(pdata->avbuffsrc_ctx, "range", color_range, AV_OPT_SEARCH_CHILDREN);
+    if (ret < 0) {
+        mlt_log_error(self, "Cannot set src range %d\n", color_range);
+        goto fail;
+    }
     ret = avfilter_init_str(pdata->avbuffsrc_ctx, NULL);
     if (ret < 0) {
         mlt_log_error(self, "Cannot init buffer source\n");
@@ -481,11 +502,21 @@ static void init_image_filtergraph(
         mlt_log_error(self, "Cannot create image buffer sink\n");
         goto fail;
     }
+#if LIBAVFILTER_VERSION_INT >= ((10 << 16) + (6 << 8) + 100)
+    ret = av_opt_set_array(pdata->avbuffsink_ctx,
+                           "pixel_formats",
+                           AV_OPT_SEARCH_CHILDREN,
+                           0,
+                           1,
+                           AV_OPT_TYPE_PIXEL_FMT,
+                           pixel_fmts);
+#else
     ret = av_opt_set_int_list(pdata->avbuffsink_ctx,
                               "pix_fmts",
                               pixel_fmts,
                               -1,
                               AV_OPT_SEARCH_CHILDREN);
+#endif
     if (ret < 0) {
         mlt_log_error(self, "Cannot set sink pixel formats\n");
         goto fail;
@@ -819,22 +850,19 @@ static int link_get_audio(mlt_frame frame,
             break;
         }
 
-        // Sanity check the output frame
 #if HAVE_FFMPEG_CH_LAYOUT
-        if (*channels != pdata->avoutframe->ch_layout.nb_channels
+        int expected_channels = pdata->avoutframe->ch_layout.nb_channels;
 #else
-        if (*channels != pdata->avoutframe->channels
+        int expected_channels = pdata->avoutframe->channels;
 #endif
-            || *samples != pdata->avoutframe->nb_samples
+
+        // Sanity check the output frame
+        if (*channels != expected_channels || *samples != pdata->avoutframe->nb_samples
             || *frequency != pdata->avoutframe->sample_rate) {
             mlt_log_error(self,
                           "Unexpected return format c %d->%d\tf %d->%d\tf %d->%d\n",
                           *channels,
-#if HAVE_FFMPEG_CH_LAYOUT
-                          pdata->avoutframe->ch_layout.nb_channels,
-#else
-                          pdata->avoutframe->channels,
-#endif
+                          expected_channels,
                           *samples,
                           pdata->avoutframe->nb_samples,
                           *frequency,
@@ -903,10 +931,14 @@ static int link_get_image(mlt_frame frame,
     mlt_service_lock(MLT_LINK_SERVICE(self));
 
     double scale = mlt_profile_scale_width(profile, *width);
+    const char *colorspace_str = mlt_properties_get(frame_properties, "colorspace");
+    mlt_colorspace colorspace = mlt_image_colorspace_id(colorspace_str);
+    int full_range = mlt_properties_get_int(frame_properties, "full_range");
 
     if (pdata->reset || pdata->format != *format || pdata->width != *width
-        || pdata->height != *height) {
-        init_image_filtergraph(self, *format, *width, *height, scale);
+        || pdata->height != *height || pdata->colorspace != colorspace
+        || pdata->full_range != full_range) {
+        init_image_filtergraph(self, *format, *width, *height, colorspace, full_range, scale);
         pdata->reset = 0;
     }
 
@@ -917,34 +949,35 @@ static int link_get_image(mlt_frame frame,
         pdata->avinframe->sample_aspect_ratio = (AVRational){profile->sample_aspect_num,
                                                              profile->sample_aspect_den};
         pdata->avinframe->pts = pos;
+#if LIBAVUTIL_VERSION_INT >= ((58 << 16) + (7 << 8) + 100)
+        if (!mlt_properties_get_int(frame_properties, "progressive"))
+            pdata->avinframe->flags |= AV_FRAME_FLAG_INTERLACED;
+        else
+            pdata->avinframe->flags &= ~AV_FRAME_FLAG_INTERLACED;
+        if (mlt_properties_get_int(frame_properties, "top_field_first"))
+            pdata->avinframe->flags |= AV_FRAME_FLAG_TOP_FIELD_FIRST;
+        else
+            pdata->avinframe->flags &= ~AV_FRAME_FLAG_TOP_FIELD_FIRST;
+#else
         pdata->avinframe->interlaced_frame = !mlt_properties_get_int(frame_properties,
                                                                      "progressive");
         pdata->avinframe->top_field_first = mlt_properties_get_int(frame_properties,
                                                                    "top_field_first");
-        pdata->avinframe->color_primaries = mlt_properties_get_int(frame_properties,
-                                                                   "color_primaries");
-        pdata->avinframe->color_trc = mlt_properties_get_int(frame_properties, "color_trc");
-        pdata->avinframe->color_range = mlt_properties_get_int(frame_properties, "full_range")
-                                            ? AVCOL_RANGE_JPEG
-                                            : AVCOL_RANGE_MPEG;
-
-        switch (mlt_properties_get_int(frame_properties, "colorspace")) {
-        case 240:
-            pdata->avinframe->colorspace = AVCOL_SPC_SMPTE240M;
-            break;
-        case 601:
-            pdata->avinframe->colorspace = AVCOL_SPC_BT470BG;
-            break;
-        case 709:
-            pdata->avinframe->colorspace = AVCOL_SPC_BT709;
-            break;
-        case 2020:
-            pdata->avinframe->colorspace = AVCOL_SPC_BT2020_NCL;
-            break;
-        case 2021:
-            pdata->avinframe->colorspace = AVCOL_SPC_BT2020_CL;
-            break;
-        }
+#endif
+        const char *primaries_str = mlt_properties_get(frame_properties, "color_primaries");
+        mlt_color_primaries primaries = mlt_image_color_pri_id(primaries_str);
+        if (primaries == mlt_color_pri_none)
+            primaries = mlt_color_primaries_from_colorspace(colorspace, *height);
+        pdata->avinframe->color_primaries = mlt_to_av_color_primaries(primaries);
+        const char *color_trc_str = mlt_properties_get(frame_properties, "color_trc");
+        mlt_color_trc trc = mlt_image_color_trc_id(color_trc_str);
+        pdata->avinframe->color_trc = mlt_to_av_color_trc(trc);
+        if (trc == mlt_color_trc_none)
+            trc = mlt_color_trc_from_colorspace(colorspace);
+        pdata->avinframe->color_range = mlt_to_av_color_range(full_range);
+        const char *colorspace_str = mlt_properties_get(frame_properties, "colorspace");
+        mlt_colorspace colorspace = mlt_image_colorspace_id(colorspace_str);
+        pdata->avinframe->colorspace = mlt_to_av_colorspace(colorspace, pdata->avinframe->height);
 
         ret = av_frame_get_buffer(pdata->avinframe, 1);
         if (ret < 0) {
