@@ -1,6 +1,6 @@
 /*
  * transition_composite.h -- compose one image over another using alpha channel
- * Copyright (C) 2003-2014 Meltytech, LLC
+ * Copyright (C) 2003-2021 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,11 +22,19 @@
 
 #include <framework/mlt_transition.h>
 
-extern mlt_transition transition_composite_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
+extern mlt_transition transition_composite_init(mlt_profile profile,
+                                                mlt_service_type type,
+                                                const char *id,
+                                                char *arg);
 
-// Courtesy functionality - allows regionalised filtering
-extern mlt_frame composite_copy_region( mlt_transition, mlt_frame, mlt_position );
-extern void composite_line_yuv( uint8_t *dest, uint8_t *src, int width, uint8_t *alpha_b,
-                                uint8_t *alpha_a, int weight, uint16_t *luma, int soft, uint32_t step );
+extern void composite_line_yuv(uint8_t *dest,
+                               uint8_t *src,
+                               int width,
+                               uint8_t *alpha_b,
+                               uint8_t *alpha_a,
+                               int weight,
+                               uint16_t *luma,
+                               int soft,
+                               uint32_t step);
 
 #endif
