@@ -22,6 +22,7 @@
 #include <frei0r.h>
 #include <string.h>
 
+#include "mltfrei0r_export.h"
 #include <dirent.h>
 #include <dlfcn.h>
 #include <limits.h>
@@ -481,7 +482,7 @@ static mlt_properties metadata(mlt_service_type type, const char *id, void *data
     return mlt_properties_parse_yaml(file);
 }
 
-MLT_REPOSITORY
+MLTFREI0R_EXPORT MLT_REPOSITORY
 {
     mlt_tokeniser tokeniser = mlt_tokeniser_init();
     char *frei0r_path = get_frei0r_path();
