@@ -11,6 +11,7 @@
 
 #include "math/bezier/bezier.hpp"
 #include "glaxnimate_format.hpp"
+#include "model/assets/assets.hpp"
 
 namespace glaxnimate::io::glaxnimate::detail {
 
@@ -132,10 +133,11 @@ public:
 
     void load_object ( model::Object* target, QJsonObject object )
     {
+        version_fixup(object);
         do_load_object(target, object, target);
     }
 
-    void load_metadata(model::Document* document, const QJsonObject& object)
+    void load_metadata(const QJsonObject& object)
     {
         document->metadata() = object["metadata"].toObject().toVariantMap();
         auto info = object["info"];
@@ -145,6 +147,40 @@ public:
             document->info().keywords.push_back(kw.toString());
     }
 
+    void load_document(QJsonObject top_level)
+    {
+        auto assets = top_level[document_version < 3 ? "defs" : "assets"].toObject();
+
+        if ( document_version < 8 )
+        {
+            QJsonObject precomps;
+            QJsonArray values;
+            if ( assets.contains("precompositions") )
+            {
+                precomps = assets["precompositions"].toObject();
+                values = precomps["values"].toArray();
+            }
+            else
+            {
+                precomps["__type__"] = "CompositionList";
+            }
+
+            if ( top_level["animation"].isObject() )
+            {
+                QJsonObject main = top_level["animation"].toObject();
+                top_level.remove("animation");
+                values.push_front(main);
+            }
+
+            precomps["values"] = values;
+            assets["precompositions"] = precomps;
+        }
+
+        load_metadata(top_level);
+        load_object(document->assets(), assets);
+        resolve();
+    }
+
 private:
     void error(const QString& msg)
     {
@@ -161,7 +197,7 @@ private:
         return fixed;
     }
 
-    void version_fixup(model::Object*, QJsonObject& object)
+    void version_fixup(QJsonObject& object)
     {
         if ( document_version == 1 )
         {
@@ -238,12 +274,43 @@ private:
             if ( object["__type__"].toString() == "MaskSettings" )
                 object["mask"] = int(object["mask"].toBool());
         }
+
+
+        if ( document_version < 8 )
+        {
+            if ( object["__type__"].toString() == "MainComposition" )
+            {
+                object["__type__"] = "Composition";
+            }
+            else if ( object["__type__"].toString() == "Precomposition" )
+            {
+                object["__type__"] = "Composition";
+                if ( !document->assets()->compositions->values.empty() )
+                {
+                    auto main = document->assets()->compositions->values[0];
+                    if ( !object.contains("fps") )
+                        object["fps"] = main->fps.get();
+                    if ( !object.contains("width") )
+                        object["width"] = main->width.get();
+                    if ( !object.contains("height") )
+                        object["height"] = main->height.get();
+                }
+            }
+            else if ( object["__type__"].toString() == "PrecompositionList" )
+            {
+                object["__type__"] = "CompositionList";
+            }
+            else if ( object["__type__"].toString() == "Assets" )
+            {
+                QJsonObject comps = object["precompositions"].toObject();
+                object.remove("precompositions");
+                object["compositions"] = comps;
+            }
+        }
     }
 
     void do_load_object ( model::Object* target, QJsonObject object, const UnresolvedPath& path )
     {
-        version_fixup(target, object);
-
         QString type = object["__type__"].toString();
 
         if ( type != target->type_name() )
@@ -440,6 +507,7 @@ private:
                 if ( !val.isObject() )
                     return {};
                 QJsonObject jobj = val.toObject();
+                version_fixup(jobj);
                 model::Object* object = create_object(jobj["__type__"].toString());
                 if ( !object )
                     return {};
@@ -534,12 +602,6 @@ private:
 
     model::Object* create_object(const QString& type)
     {
-        if ( type == "MainComposition" )
-        {
-            error(GlaxnimateFormat::tr("Objects of type 'MainComposition' can only be at the top level of the document"));
-            return nullptr;
-        }
-
         if ( auto obj = model::Factory::instance().build(type, document) )
         {
             temporaries.emplace_back(obj);
