@@ -23,14 +23,13 @@ public:
     QStringList extensions() const override { return {"json"}; }
     bool can_save() const override { return true; }
     bool can_open() const override { return true; }
-    std::unique_ptr<app::settings::SettingsGroup> save_settings(model::Document*) const override;
+    std::unique_ptr<app::settings::SettingsGroup> save_settings(model::Composition*) const override;
 
-    QCborMap to_json(model::Document* document, bool strip = false, bool strip_raster = false, const QVariantMap& settings = {});
+    QCborMap to_json(model::Composition* comp, bool strip = false, bool strip_raster = false, const QVariantMap& settings = {});
     bool load_json(const QByteArray& data, model::Document* document);
 
 private:
-    bool on_save(QIODevice& file, const QString& filename,
-                 model::Document* document, const QVariantMap& setting_values) override;
+    bool on_save(QIODevice& file, const QString& filename, model::Composition* comp, const QVariantMap& setting_values) override;
 
     bool on_open(QIODevice& file, const QString& filename,
                  model::Document* document, const QVariantMap& setting_values) override;
