@@ -1,3 +1,9 @@
+/*
+ * SPDX-FileCopyrightText: 2019-2023 Mattia Basaglia <dev@dragon.best>
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
+ */
+
 #pragma once
 
 #include <set>
@@ -186,6 +192,11 @@ public:
      */
     math::bezier::Bezier removed_points(const std::set<int>& indices) const;
 
+    /**
+     * \brief For closed beziers, ensure the last segment is present
+     */
+    void add_close_point();
+
 private:
     /**
      * \brief Solver for the point \p p to the point \p p + 1
@@ -297,4 +308,13 @@ private:
 
 } // namespace glaxnimate::math
 
+namespace glaxnimate::math {
+
+inline bezier::Bezier lerp(const math::bezier::Bezier& a, const math::bezier::Bezier& b, qreal factor)
+{
+    return a.lerp(b, factor);
+}
+
+} // namespace glaxnimate::math
+
 Q_DECLARE_METATYPE(glaxnimate::math::bezier::Bezier)
