@@ -17,6 +17,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "mltkdenlive_export.h"
 #include <framework/mlt.h>
 #include <limits.h>
 #include <string.h>
@@ -45,7 +46,7 @@ static mlt_properties metadata(mlt_service_type type, const char *id, void *data
     return mlt_properties_parse_yaml(file);
 }
 
-MLT_REPOSITORY
+MLTKDENLIVE_EXPORT MLT_REPOSITORY
 {
     MLT_REGISTER(mlt_service_filter_type, "boxblur", filter_boxblur_init);
     MLT_REGISTER(mlt_service_filter_type, "freeze", filter_freeze_init);
