@@ -17,22 +17,27 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <string.h>
-#include <limits.h>
+#include "mltopencv_export.h"
 #include <framework/mlt.h>
+#include <limits.h>
+#include <string.h>
+extern mlt_filter filter_tracker_init(mlt_profile profile,
+                                      mlt_service_type type,
+                                      const char *id,
+                                      char *arg);
 
-
-extern mlt_filter filter_tracker_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
-
-static mlt_properties metadata( mlt_service_type type, const char *id, void *data )
+static mlt_properties metadata(mlt_service_type type, const char *id, void *data)
 {
-	char file[ PATH_MAX ];
-	snprintf( file, PATH_MAX, "%s/opencv/%s", mlt_environment( "MLT_DATA" ), (char*) data );
-	return mlt_properties_parse_yaml( file );
+    char file[PATH_MAX];
+    snprintf(file, PATH_MAX, "%s/opencv/%s", mlt_environment("MLT_DATA"), (char *) data);
+    return mlt_properties_parse_yaml(file);
 }
 
-MLT_REPOSITORY
+MLTOPENCV_EXPORT MLT_REPOSITORY
 {
-	MLT_REGISTER( filter_type, "opencv.tracker", filter_tracker_init );
-	MLT_REGISTER_METADATA( filter_type, "opencv.tracker", metadata, "filter_opencv_tracker.yml" );
+    MLT_REGISTER(mlt_service_filter_type, "opencv.tracker", filter_tracker_init);
+    MLT_REGISTER_METADATA(mlt_service_filter_type,
+                          "opencv.tracker",
+                          metadata,
+                          "filter_opencv_tracker.yml");
 }
