@@ -30,7 +30,8 @@
 //shifts all (longitude) values from near 180 to 0
 double get_180_swapped(double lon)
 {
-    // mlt_log_info(NULL, "get_180_swapped(%f) -> %f\n", lon, lon + ( lon>0 ? -180 : 180));
+    if (lon == GPS_UNINIT)
+        return lon;
     return lon + (lon > 0 ? -180 : 180);
 }
 
@@ -124,7 +125,7 @@ double distance_haversine_2p(double p1_lat, double p1_lon, double p2_lat, double
     return R * c; //return is in meters because radius is in meters
 }
 
-// Returns the distance between 2 gps points, accurate enough (<0.1% error) for distances below 4km
+// Returns the distance between 2 gps points (uses haversine formula if needed)
 double distance_equirectangular_2p(double p1_lat, double p1_lon, double p2_lat, double p2_lon)
 {
     //use the haversine formula for very far points
@@ -332,9 +333,13 @@ int binary_search_gps(gps_private_data gdata, int64_t video_time, bool force_res
 
 /** Returns a nicer number of decimal values for floats
  *  [ 1.23m | 12.3m | 123m ]
+ *  - argument use_decimals defaults to -1 (in .h)
 */
-int decimals_needed(double x)
+int decimals_needed(double x, int use_decimals)
 {
+    if (use_decimals != -1)
+        return use_decimals;
+
     if (fabs(x) < 10)
         return 2;
     if (fabs(x) < 100)
@@ -346,7 +351,7 @@ int decimals_needed(double x)
 */
 double convert_distance_to_format(double x, const char *format)
 {
-    if (format == NULL)
+    if (format == NULL || x == GPS_UNINIT)
         return x;
 
     if (strstr(format, "km") || strstr(format, "kilometer"))
@@ -364,17 +369,23 @@ double convert_distance_to_format(double x, const char *format)
 */
 double convert_speed_to_format(double x, const char *format)
 {
+    if (x == GPS_UNINIT)
+        return x;
+    //order is important as short keywords will match anywhere (ms in kms and mi in mi[nutes])
     if (format == NULL || strstr(format, "kms") || strstr(format, "km/s")
         || strstr(format, "kilometer"))
         return x * 3.6; //default km/h
-
     if (strstr(format, "ms") || strstr(format, "m/s") || strstr(format, "meter"))
         return x;
-    else if (strstr(format, "mi") || strstr(format, "mi/h") || strstr(format, "mile"))
+    if (strstr(format, "mmin") || strstr(format, "m/min"))
+        return x * 60;
+    if (strstr(format, "ftmin") || strstr(format, "ft/min"))
+        return x * 196.850393;
+    if (strstr(format, "mi") || strstr(format, "mi/h") || strstr(format, "mile"))
         return x * 2.23693629;
-    else if (strstr(format, "kn") || strstr(format, "nm/h") || strstr(format, "knots"))
+    if (strstr(format, "kn") || strstr(format, "nm/h") || strstr(format, "knots"))
         return x * 1.94384449;
-    else if (strstr(format, "ft") || strstr(format, "ft/s") || strstr(format, "feet"))
+    if (strstr(format, "ft") || strstr(format, "ft/s") || strstr(format, "feet"))
         return x * 3.2808399;
 
     return x * 3.6;
@@ -385,6 +396,9 @@ double convert_speed_to_format(double x, const char *format)
  */
 const char *bearing_to_compass(double x)
 {
+    if (x == GPS_UNINIT)
+        return "--";
+
     if (x <= 22.5 || x >= 360 - 22.5)
         return "N";
     else if (x < 45 + 22.5)
@@ -448,6 +462,10 @@ void recalculate_gps_data(gps_private_data gdata)
            start_dist_up = 0, start_dist_down = 0, start_dist_flat = 0;
     gps_point_proc *crt_point = NULL, *prev_point = NULL, *prev_nrsmooth_point = NULL;
 
+    int grade_bucket[100], speed_avg_count = 0;
+    double speed_avg = 0, last_grade = 0;
+    memset(&grade_bucket, 0, 100 * sizeof(int));
+
     for (i = 0; i < gps_points_size; i++) {
         //store values at processing_start_time to substract them at the end
         if (i - 1 == offset_start) {
@@ -491,17 +509,22 @@ void recalculate_gps_data(gps_private_data gdata)
             smooth_index++;
         if (smooth_index < i)
             prev_nrsmooth_point = &gps_points[smooth_index];
+        else
+            prev_nrsmooth_point = NULL;
 
         //1) get distance difference between last 2 valid points
         double d_dist = distance_equirectangular_2p(prev_point->lat,
                                                     prev_point->lon,
                                                     crt_point->lat,
                                                     crt_point->lon);
+        double d_elev = 0, d_dist_smoothed = 0, d_elev_smoothed = 0;
+        //the int64 diff will be small enough for double
+        double d_time_sec = (crt_point->time - prev_point->time) / 1000.0;
+        double d_time_smoothed_sec = d_time_sec;
 
         //if time difference is way longer for one point than usual, don't do math on that gap (treat recording paused, move far, then resume)
-        //5*average time
-        if ((crt_point->time - prev_point->time)
-            > 5.0 * (*gdata.last_gps_time - *gdata.first_gps_time) / *gdata.gps_points_size) {
+        if (d_time_sec
+            > 10.0 * (*gdata.last_gps_time - *gdata.first_gps_time) / *gdata.gps_points_size) {
             prev_nrsmooth_point = NULL;
             ignore_points_before = i;
             crt_point->total_dist = total_dist;
@@ -513,28 +536,38 @@ void recalculate_gps_data(gps_private_data gdata)
         total_dist += d_dist;
         crt_point->total_dist = total_dist;
 
+        //distance and height difference if smoothed point can be used
+        if (prev_nrsmooth_point) {
+            d_time_smoothed_sec = (crt_point->time - prev_nrsmooth_point->time) / 1000.0;
+            d_dist_smoothed = crt_point->total_dist - prev_nrsmooth_point->total_dist;
+            if (crt_point->ele != GPS_UNINIT && prev_nrsmooth_point->ele != GPS_UNINIT)
+                d_elev_smoothed = crt_point->ele - prev_nrsmooth_point->ele;
+        }
+
         //2)+3) speed and bearing
         if (req_smooth < 2) {
-            crt_point->speed = d_dist / ((crt_point->time - prev_point->time) / 1000.0); //in m/s
+            crt_point->speed = d_dist / d_time_sec; //in m/s
             crt_point->bearing
                 = bearing_2p(prev_point->lat, prev_point->lon, crt_point->lat, crt_point->lon);
-        } else //for "smoothing" we calculate distance between 2 points "nr_smoothing" distance behind
-        {
-            if (prev_nrsmooth_point) {
-                double d_dist = crt_point->total_dist - prev_nrsmooth_point->total_dist;
-                int64_t d_time = crt_point->time - prev_nrsmooth_point->time;
-                crt_point->speed = d_dist / (d_time / 1000.0);
+        } else if (prev_nrsmooth_point) {
+            //for "smoothing" we calculate distance between 2 points "nr_smoothing" distance behind
+            crt_point->speed = d_dist_smoothed / d_time_smoothed_sec;
 
-                crt_point->bearing = bearing_2p(prev_nrsmooth_point->lat,
-                                                prev_nrsmooth_point->lon,
-                                                crt_point->lat,
-                                                crt_point->lon);
-            }
+            crt_point->bearing = bearing_2p(prev_nrsmooth_point->lat,
+                                            prev_nrsmooth_point->lon,
+                                            crt_point->lat,
+                                            crt_point->lon);
+        }
+
+        //continuously compute speed average
+        if (crt_point->speed > 0.27) {
+            speed_avg = (speed_avg * speed_avg_count + crt_point->speed) / (speed_avg_count + 1);
+            ++speed_avg_count;
         }
 
         //4) altitude stuff
         if (crt_point->ele != GPS_UNINIT && prev_point->ele != GPS_UNINIT) {
-            double d_elev = crt_point->ele - prev_point->ele;
+            d_elev = crt_point->ele - prev_point->ele;
             total_d_elev += d_elev;
             if (crt_point->ele > prev_point->ele) {
                 total_elev_up += d_elev;
@@ -542,9 +575,9 @@ void recalculate_gps_data(gps_private_data gdata)
             } else if (crt_point->ele < prev_point->ele) {
                 total_elev_down += d_elev;
                 total_dist_down += d_dist;
-            } else
+            } else {
                 total_dist_flat += d_dist;
-
+            }
             crt_point->d_elev = total_d_elev;
             crt_point->elev_up = total_elev_up;
             crt_point->elev_down = total_elev_down;
@@ -553,9 +586,70 @@ void recalculate_gps_data(gps_private_data gdata)
             crt_point->dist_flat = total_dist_flat;
         }
 
+        //5) grade, vertical and 3d speed (if altitude present)
+        if (crt_point->ele != GPS_UNINIT) {
+            bool ok = 1;
+            double used_d_elev = d_elev, used_d_dist = d_dist, used_d_time_sec = d_time_sec;
+            if (prev_nrsmooth_point && prev_nrsmooth_point->ele != GPS_UNINIT) {
+                used_d_elev = d_elev_smoothed;
+                used_d_dist = d_dist_smoothed;
+                used_d_time_sec = d_time_smoothed_sec;
+            } else if (prev_point->ele == GPS_UNINIT) {
+                //this looks cleaner; no previous elevation available
+                ok = 0;
+            }
+
+            if (ok) {
+                //cut-off grade computation (->just duplicate it) under 2km/h and half the avg speed (but don't cut-off over 15km/h)
+                if (used_d_dist > 0.28 * (req_smooth + 1) //this covers division by 0
+                    && used_d_dist > MIN(speed_avg / 2, 4) * req_smooth) {
+                    crt_point->grade_p = 100.0 * used_d_elev / used_d_dist;
+                    int indx = CLAMP(abs(crt_point->grade_p) / 10, 0, 99);
+                    grade_bucket[indx]++;
+                    last_grade = crt_point->grade_p;
+                } else {
+                    //better option: set this to GPS_UNINIT and run an interpolation at the end to re-fill
+                    crt_point->grade_p = last_grade / 2;
+                }
+                //vertical + 3d speed
+                crt_point->speed_vertical = used_d_elev / used_d_time_sec;
+                crt_point->speed_3d = sqrt(used_d_dist * used_d_dist + used_d_elev * used_d_elev)
+                                      / used_d_time_sec;
+            }
+        }
         prev_point = crt_point;
     }
 
+    //for gps grade only: trim down the highest 1st percentile of values
+    if (req_smooth > 1) {
+        int nr_1p = gps_points_size / 100;
+        int max_grade_1p = -1;
+        for (i = 99; i > 1; --i) {
+            if (grade_bucket[i] > nr_1p) {
+                max_grade_1p = (i + 1) * 10;
+                break;
+            }
+        }
+        if (max_grade_1p > 1) {
+            for (i = 0; i < gps_points_size; i++) {
+                if (gps_points[i].grade_p != GPS_UNINIT
+                    && abs(gps_points[i].grade_p) > max_grade_1p) {
+                    //if speed is over 15km/h assume grade is correct
+                    if (gps_points[i].speed < MAX(speed_avg / 2, 4)) {
+                        // mlt_log_info(NULL,
+                        //              "clamping [%d].grade_p from %f to %d; speed = %f",
+                        //              i,
+                        //              gps_points[i].grade_p,
+                        //              max_grade_1p,
+                        //              gps_points[i].speed);
+                        gps_points[i].grade_p = (gps_points[i].grade_p >= 0 ? max_grade_1p
+                                                                            : -max_grade_1p);
+                    }
+                }
+            }
+        }
+    }
+
     //clean up computed values for relative stuff (distances mostly) before gps_processing_start time
     if (gdata.gps_proc_start_t != 0 && offset_start > 0 && offset_start < gps_points_size) {
         //mlt_log_info(gdata.filter, "recalculate_gps_data: clearing gps data from 0 to %d due to set GPS start time:%d s", offset_start, gdata.gps_proc_start_t/1000);
@@ -650,6 +744,18 @@ gps_point_proc weighted_middle_point_proc(gps_point_proc *p1,
         = weighted_middle_double(p1->lon, p1->time, p2->lon, p2->time, new_t, max_gps_diff_ms);
     crt_point.speed
         = weighted_middle_double(p1->speed, p1->time, p2->speed, p2->time, new_t, max_gps_diff_ms);
+    crt_point.speed_vertical = weighted_middle_double(p1->speed_vertical,
+                                                      p1->time,
+                                                      p2->speed_vertical,
+                                                      p2->time,
+                                                      new_t,
+                                                      max_gps_diff_ms);
+    crt_point.speed_3d = weighted_middle_double(p1->speed_3d,
+                                                p1->time,
+                                                p2->speed_3d,
+                                                p2->time,
+                                                new_t,
+                                                max_gps_diff_ms);
     crt_point.total_dist = weighted_middle_double(p1->total_dist,
                                                   p1->time,
                                                   p2->total_dist,
@@ -700,6 +806,18 @@ gps_point_proc weighted_middle_point_proc(gps_point_proc *p1,
                                                max_gps_diff_ms);
     crt_point.hr
         = weighted_middle_double(p1->hr, p1->time, p2->hr, p2->time, new_t, max_gps_diff_ms);
+    crt_point.cad
+        = weighted_middle_double(p1->cad, p1->time, p2->cad, p2->time, new_t, max_gps_diff_ms);
+    crt_point.grade_p = weighted_middle_double(p1->grade_p,
+                                               p1->time,
+                                               p2->grade_p,
+                                               p2->time,
+                                               new_t,
+                                               max_gps_diff_ms);
+    crt_point.atemp
+        = weighted_middle_double(p1->atemp, p1->time, p2->atemp, p2->time, new_t, max_gps_diff_ms);
+    crt_point.power
+        = weighted_middle_double(p1->power, p1->time, p2->power, p2->time, new_t, max_gps_diff_ms);
     return crt_point;
 }
 
@@ -740,23 +858,31 @@ void process_gps_smoothing(gps_private_data gdata, char do_processing)
     }
 
     int max_gps_diff_ms = get_max_gps_diff_ms(gdata);
-    int i, j, nr_hr = 0, nr_ele = 0;
-    double hr = GPS_UNINIT;
-    double ele = GPS_UNINIT;
+    int i, j, nr_hr = 0, nr_ele = 0, nr_cad = 0, nr_atemp = 0, nr_power = 0;
+    double hr = GPS_UNINIT, ele = GPS_UNINIT, cad = GPS_UNINIT, atemp = GPS_UNINIT,
+           power = GPS_UNINIT;
 
-    //linear interpolation for heart rate and elevation, one time per file, ignores start offset
+    //linear interpolation for heart rate, elevation, cadence and temperature, one time per file, ignores start offset
     if (*gdata.interpolated == 0) {
-        //NOTE: interpolation limit is now 60 values = arbitrarily chosen @ 1min (if 1sec interval), find a better value?
+        //figure out how many seconds are between 2 average fixes so we can set a limit (in time) to interpolation
+        double avg_time = (*gdata.last_gps_time - *gdata.first_gps_time) / 1000
+                          / (double) *gdata.gps_points_size;
+        double nr_one_minute = 60.0 / (avg_time ? avg_time : 1);
+
         gps_point_raw *gp_r = gdata.gps_points_r;
         gps_point_proc *gp_p = gdata.gps_points_p;
         for (i = 0; i < *gdata.gps_points_size; i++) {
-            gp_p[i].hr = gp_r[i].hr; //calloc made everything 0, fill back with GPS_UNINIT if needed
+            //calloc made everything 0, fill back with GPS_UNINIT if needed
+            gp_p[i].hr = gp_r[i].hr;
             gp_p[i].ele = gp_r[i].ele;
+            gp_p[i].cad = gp_r[i].cad;
+            gp_p[i].atemp = gp_r[i].atemp;
+            gp_p[i].power = gp_r[i].power;
 
             //heart rate
             if (gp_r[i].hr != GPS_UNINIT) { //found valid hr
                 if (hr != GPS_UNINIT && nr_hr > 0
-                    && nr_hr <= 60) { //there were missing values and had a hr before
+                    && nr_hr <= nr_one_minute) { //there were missing values and had a hr before
                     nr_hr++;
                     for (j = i; j > i - nr_hr; j--) { //go backwards and fill values
                         gp_p[j].hr = hr + (gp_r[i].hr - hr) * (1.0 * (j - (i - nr_hr)) / nr_hr);
@@ -770,7 +896,7 @@ void process_gps_smoothing(gps_private_data gdata, char do_processing)
 
             //altitude
             if (gp_r[i].ele != GPS_UNINIT) {
-                if (ele != GPS_UNINIT && nr_ele > 0 && nr_ele <= 60) {
+                if (ele != GPS_UNINIT && nr_ele > 0 && nr_ele <= nr_one_minute * 10) {
                     nr_ele++;
                     for (j = i; j > i - nr_ele; j--) {
                         gp_p[j].ele = ele
@@ -784,6 +910,53 @@ void process_gps_smoothing(gps_private_data gdata, char do_processing)
             } else
                 nr_ele++;
 
+            //cadence
+            if (gp_r[i].cad != GPS_UNINIT) {
+                if (cad != GPS_UNINIT && nr_cad > 0 && nr_cad <= nr_one_minute) {
+                    nr_cad++;
+                    for (j = i; j > i - nr_cad; j--) {
+                        gp_p[j].cad = cad
+                                      + 1.0 * (gp_r[i].cad - cad)
+                                            * (1.0 * (j - (i - nr_cad)) / nr_cad);
+                    }
+                }
+                cad = gp_r[i].cad;
+                nr_cad = 0;
+            } else
+                nr_cad++;
+
+            //temperature
+            if (gp_r[i].atemp != GPS_UNINIT) {
+                if (atemp != GPS_UNINIT && nr_atemp > 0 && nr_atemp <= nr_one_minute * 60) {
+                    nr_atemp++;
+                    for (j = i; j > i - nr_atemp; j--) {
+                        gp_p[j].atemp = atemp
+                                        + 1.0 * (gp_r[i].atemp - atemp)
+                                              * (1.0 * (j - (i - nr_atemp)) / nr_atemp);
+                    }
+                }
+                atemp = gp_r[i].atemp;
+                nr_atemp = 0;
+            } else {
+                nr_atemp++;
+            }
+
+            //power
+            if (gp_r[i].power != GPS_UNINIT) {
+                if (power != GPS_UNINIT && nr_power > 0 && nr_power <= nr_one_minute) {
+                    nr_power++;
+                    for (j = i; j > i - nr_power; j--) {
+                        gp_p[j].power = power
+                                        + 1.0 * (gp_r[i].power - power)
+                                              * (1.0 * (j - (i - nr_power)) / nr_power);
+                    }
+                }
+                power = gp_r[i].power;
+                nr_power = 0;
+            } else {
+                nr_power++;
+            }
+
             //these are not interpolated but as long as we're iterating we can copy them now
             gp_p[i].time = gp_r[i].time;
             gp_p[i].lat = gp_r[i].lat;
@@ -796,9 +969,8 @@ void process_gps_smoothing(gps_private_data gdata, char do_processing)
     const int gps_points_size = *gdata.gps_points_size;
 
     for (i = 0; i < gps_points_size; i++) {
-        if (req_smooth
-            == 1) //copy raw lat/lon to calc lat/lon and interpolate 1 location if necessary
-        {
+        if (req_smooth == 1) {
+            //copy raw lat/lon to calc lat/lon and interpolate 1 location if necessary
             gps_points_p[i].lat = gps_points_r[i].lat;
             gps_points_p[i].lon = gps_points_r[i].lon;
 
@@ -826,12 +998,12 @@ void process_gps_smoothing(gps_private_data gdata, char do_processing)
 
                 //mlt_log_info(gdata.filter, "interpolating position for smooth=1, new point[%d]:%f,%f @time=%d s", i, gps_points_p[i].lat, gps_points_p[i].lon, gps_points_r[i].time/1000);
             }
-        } else if (req_smooth > 1) //for each point average "req_smooth/2" values before and after
-        {
+        } else if (req_smooth > 1) {
+            //for each point average "req_smooth/2" values before and after
             double lat_sum = 0, lon_sum = 0;
             int nr_div = 0;
 
-            //TODO: treat case of points on opposite sides of 180*
+            //passing 180 meridian is ok, but passing both 180 and 0 (so more than one full circle around the earth) will smooth out badly; no fix planned for this
             for (j = MAX(0, i - req_smooth / 2); j < MIN(i + req_smooth / 2, gps_points_size); j++) {
                 if (has_valid_location(gps_points_r[j]) && in_gps_time_window(gdata, i, j)) {
                     lat_sum += gps_points_r[j].lat;
@@ -860,24 +1032,30 @@ void process_gps_smoothing(gps_private_data gdata, char do_processing)
 void qxml_parse_gpx(QXmlStreamReader &reader, gps_point_ll **gps_list, int *count_pts)
 {
     int64_t last_time = -1;
+    //support no time in file because many services (Strava included) do this for exported routes
+    gps_point_ll *no_time_head = NULL, *no_time_prev = NULL;
+    int no_time_count = 0;
     /*
-	// sample point from .GPX file (version 1.0) + HR extension
-	<trkpt lat="45.227621666484104" lon="25.16533185322889">
-		<ele>868.3005248873908</ele>
-		<time>2020-07-11T09:03:23.000Z</time>
-		<speed>0.0</speed>
-		<course>337.1557</course>
-		<extensions>
-		 <gpxtpx:TrackPointExtension>
-		  <gpxtpx:hr>132</gpxtpx:hr>
-		 </gpxtpx:TrackPointExtension>
-		</extensions>
-	</trkpt>
-	*/
+    // sample point from .GPX file (version 1.0) + TrackPointExtension
+    <trkpt lat="45.227621666484104" lon="25.16533185322889">
+        <ele>868.3005248873908</ele>
+        <time>2020-07-11T09:03:23.000Z</time>
+        <speed>0.0</speed>
+        <course>337.1557</course>
+        <extensions>
+            <power>0</power>                      //strava ...
+            <gpxtpx:TrackPointExtension>          //official garmin extension
+                <gpxtpx:hr>132</gpxtpx:hr>            //heart rate in beats per minute
+                <gpxtpx:cad>75</gpxtpx:cad>           //cadence in revolutions per minute
+                <gpxtpx:atemp>27.5</gpxtpx:atemp>     //ambient temperature in degrees celsius
+            </gpxtpx:TrackPointExtension>
+        </extensions>
+    </trkpt>
+    */
     while (!reader.atEnd() && !reader.hasError()) {
         reader.readNext();
         // mlt_log_info(NULL, " readNext tag: %s type = %d\n", _qxml_getthestring(reader.name()), reader.tokenType());
-        if (reader.isStartElement() && reader.name() == QString("trkpt")) {
+        if (reader.isStartElement() && reader.name() == QStringLiteral("trkpt")) {
             gps_point_raw crt_point = uninit_gps_raw_point;
 
             QXmlStreamAttributes attributes = reader.attributes();
@@ -887,28 +1065,40 @@ void qxml_parse_gpx(QXmlStreamReader &reader, gps_point_ll **gps_list, int *coun
                 crt_point.lon = attributes.value("lon").toDouble();
 
             while (reader.readNext()
-                   && !(reader.name() == QString("trkpt")
+                   && !(reader.name() == QStringLiteral("trkpt")
                         && reader.tokenType() == QXmlStreamReader::EndElement)) //until closing trkpt
             {
                 // mlt_log_info(NULL, "[trkpt->1] readNext tag: %s, type=%d \n", _qxml_getthestring(reader.name()), reader.tokenType());
                 if (!reader.isStartElement())
                     continue;
 
-                if (reader.name() == QString("ele"))
+                if (reader.name() == QStringLiteral("ele"))
                     crt_point.ele = reader.readElementText().toDouble();
-                else if (reader.name() == QString("time"))
+                else if (reader.name() == QStringLiteral("time"))
                     crt_point.time = datetimeXMLstring_to_mseconds(
                         qUtf8Printable(reader.readElementText()));
-                else if (reader.name() == QString("speed"))
+                else if (reader.name() == QStringLiteral("speed"))
                     crt_point.speed = reader.readElementText().toDouble();
-                else if (reader.name() == QString("course"))
+                else if (reader.name() == QStringLiteral("course"))
                     crt_point.bearing = reader.readElementText().toDouble();
-                else if (reader.name() == QString("extensions")) {
-                    reader.readNextStartElement();
-                    if (reader.name() == QString("TrackPointExtension")) {
-                        reader.readNextStartElement();
-                        if (reader.name() == QString("hr"))
-                            crt_point.hr = reader.readElementText().toDouble();
+                else if (reader.name() == QStringLiteral("extensions")) {
+                    while (reader.readNext()
+                           && !(reader.name() == QStringLiteral("extensions")
+                                && reader.tokenType() == QXmlStreamReader::EndElement)) {
+                        if (reader.name() == QStringLiteral("power"))
+                            crt_point.power = reader.readElementText().toDouble();
+                        else if (reader.name() == QStringLiteral("TrackPointExtension")) {
+                            while (reader.readNext()
+                                   && !(reader.name() == QStringLiteral("TrackPointExtension")
+                                        && reader.tokenType() == QXmlStreamReader::EndElement)) {
+                                if (reader.name() == QStringLiteral("hr"))
+                                    crt_point.hr = reader.readElementText().toDouble();
+                                else if (reader.name() == QStringLiteral("cad"))
+                                    crt_point.cad = reader.readElementText().toDouble();
+                                else if (reader.name() == QStringLiteral("atemp"))
+                                    crt_point.atemp = reader.readElementText().toDouble();
+                            }
+                        }
                     }
                 }
             }
@@ -923,7 +1113,23 @@ void qxml_parse_gpx(QXmlStreamReader &reader, gps_point_ll **gps_list, int *coun
                 (*gps_list)->next = NULL;
                 gps_list = &(*gps_list)->next;
                 last_time = crt_point.time;
-            } else
+            } else if (crt_point.time == GPS_UNINIT) {
+                //if no time is stored, we make it up starting from 1
+                gps_point_ll *tmp = (gps_point_ll *) calloc(1, sizeof(gps_point_ll));
+                if (tmp == NULL)
+                    return;
+                if (no_time_head == NULL) {
+                    no_time_head = tmp;
+                    no_time_prev = tmp;
+                } else {
+                    no_time_prev->next = tmp;
+                    no_time_prev = tmp;
+                }
+                no_time_count += 1;
+                crt_point.time = no_time_count * 1000;
+                tmp->gp = crt_point;
+                tmp->next = NULL;
+            } else {
                 mlt_log_info(NULL,
                              "qxml_parse_gpx: skipping point due to time [%d] %f,%f - crt:%u.%u, "
                              "last:%u.%u\n",
@@ -934,6 +1140,27 @@ void qxml_parse_gpx(QXmlStreamReader &reader, gps_point_ll **gps_list, int *coun
                              (unsigned) (crt_point.time % 1000),
                              (unsigned) (last_time / 1000),
                              (unsigned) (last_time % 1000));
+            }
+        }
+    }
+    //if no points had time, we replace the real list with the no_time one
+    if (*count_pts == 0 && no_time_count > 0) {
+        mlt_log_info(NULL,
+                     "qxml_parse_gpx: all GPS points are missing time values, inserting each "
+                     "point at 1 second interval!");
+        *gps_list = no_time_head;
+        *count_pts = no_time_count;
+    } else if (*count_pts > 0 && no_time_count > 0) {
+        mlt_log_info(NULL,
+                     "qxml_parse_gpx: %d GPS points have no time in original file and have been "
+                     "discarded (total valid points kept: %d)!",
+                     no_time_count,
+                     *count_pts);
+        //cleanup the no_time list as we're not using it
+        while (no_time_head) {
+            gps_point_ll *tmp = no_time_head;
+            no_time_head = no_time_head->next;
+            free(tmp);
         }
     }
 }
@@ -942,30 +1169,40 @@ void qxml_parse_gpx(QXmlStreamReader &reader, gps_point_ll **gps_list, int *coun
 void qxml_parse_tcx(QXmlStreamReader &reader, gps_point_ll **gps_list, int *count_pts)
 {
     int64_t last_time = -1;
-
+    //support no time in file because many services (Strava included) do this for exported routes
+    gps_point_ll *no_time_head = NULL, *no_time_prev = NULL;
+    int no_time_count = 0;
     /*
-	//sample point from .TCX file
-	  <Trackpoint>
-		<Time>2021-02-27T14:03:02+00:00</Time>
-		<Position>
-		  <LatitudeDegrees>44.48205950925148</LatitudeDegrees>
-		  <LongitudeDegrees>26.1843781367422</LongitudeDegrees>
-		</Position>
-		<AltitudeMeters>65.0</AltitudeMeters>
-		<DistanceMeters>25919.213486999884</DistanceMeters>
-		<HeartRateBpm xsi:type="HeartRateInBeatsPerMinute_t">
-		  <Value>124</Value>
-		</HeartRateBpm>
-	  </Trackpoint>
-	*/
+    //sample point from .TCX file
+    <Trackpoint>
+        <Time>2021-02-27T14:03:02+00:00</Time>
+        <Position>
+            <LatitudeDegrees>44.48205950925148</LatitudeDegrees>
+            <LongitudeDegrees>26.1843781367422</LongitudeDegrees>
+        </Position>
+        <AltitudeMeters>65.0</AltitudeMeters>
+        <DistanceMeters>25919.213486999884</DistanceMeters>
+        <HeartRateBpm xsi:type="HeartRateInBeatsPerMinute_t">
+            <Value>124</Value>
+        </HeartRateBpm>
+        <Cadence>78</Cadence>                   //unofficial, but some popular converters use it
+        <Temperature>25</Temperature>           //unofficial/not widely used
+        <Extensions>                            //garmin's extension for power
+            <ns3:TPX>
+                <ns3:Speed>0.3449999988079071</ns3:Speed>
+                <ns3:Watts>0</ns3:Watts>
+            </ns3:TPX>
+        </Extensions>
+    </Trackpoint>
+    */
     while (!reader.atEnd() && !reader.hasError()) {
         reader.readNext();
         // mlt_log_info(NULL, " readNextStartElement tag: %s \n", _qxml_getthestring(reader.name()));
-        if (reader.isStartElement() && reader.name() == QString("Trackpoint")) {
+        if (reader.isStartElement() && reader.name() == QStringLiteral("Trackpoint")) {
             gps_point_raw crt_point = uninit_gps_raw_point;
             while (
                 reader.readNext()
-                && !(reader.name() == QString("Trackpoint")
+                && !(reader.name() == QStringLiteral("Trackpoint")
                      && reader.tokenType()
                             == QXmlStreamReader::EndElement)) //loop until we hit the closing </Trackpoint>
             {
@@ -973,24 +1210,38 @@ void qxml_parse_tcx(QXmlStreamReader &reader, gps_point_ll **gps_list, int *coun
                 if (!reader.isStartElement())
                     continue;
 
-                if (reader.name() == QString("Time"))
+                if (reader.name() == QStringLiteral("Time"))
                     crt_point.time = datetimeXMLstring_to_mseconds(
                         qUtf8Printable(reader.readElementText()));
-                else if (reader.name() == QString("Position")) {
+                else if (reader.name() == QStringLiteral("Position")) {
                     reader.readNextStartElement();
-                    if (reader.name() == QString("LatitudeDegrees"))
+                    if (reader.name() == QStringLiteral("LatitudeDegrees"))
                         crt_point.lat = reader.readElementText().toDouble();
                     reader.readNextStartElement();
-                    if (reader.name() == QString("LongitudeDegrees"))
+                    if (reader.name() == QStringLiteral("LongitudeDegrees"))
                         crt_point.lon = reader.readElementText().toDouble();
-                } else if (reader.name() == QString("AltitudeMeters")) {
+                } else if (reader.name() == QStringLiteral("AltitudeMeters")) {
                     crt_point.ele = reader.readElementText().toDouble();
-                } else if (reader.name() == QString("DistanceMeters")) {
+                } else if (reader.name() == QStringLiteral("DistanceMeters")) {
                     crt_point.total_dist = reader.readElementText().toDouble();
-                } else if (reader.name() == QString("HeartRateBpm")) {
+                } else if (reader.name() == QStringLiteral("HeartRateBpm")) {
                     reader.readNextStartElement();
-                    if (reader.name() == QString("Value"))
+                    if (reader.name() == QStringLiteral("Value"))
                         crt_point.hr = reader.readElementText().toDouble();
+                } else if (reader.name() == QStringLiteral("Cadence")) {
+                    crt_point.cad = reader.readElementText().toDouble();
+                } else if (reader.name() == QStringLiteral("Temperature")) {
+                    crt_point.atemp = reader.readElementText().toDouble();
+                } else if (reader.isStartElement()
+                           && reader.name() == QStringLiteral("Extensions")) {
+                    while (reader.readNext()
+                           && !(reader.name() == QStringLiteral("TPX")
+                                && reader.tokenType() == QXmlStreamReader::EndElement)) {
+                        if (reader.name() == QStringLiteral("Speed"))
+                            crt_point.speed = reader.readElementText().toDouble();
+                        else if (reader.name() == QStringLiteral("Watts"))
+                            crt_point.power = reader.readElementText().toDouble();
+                    }
                 }
             }
             //now add the point to linked list (but only if increasing time)
@@ -1004,6 +1255,22 @@ void qxml_parse_tcx(QXmlStreamReader &reader, gps_point_ll **gps_list, int *coun
                 (*gps_list)->next = NULL;
                 gps_list = &(*gps_list)->next;
                 last_time = crt_point.time;
+            } else if (crt_point.time == GPS_UNINIT) {
+                //if no time is stored, we make it up starting from 1
+                gps_point_ll *tmp = (gps_point_ll *) calloc(1, sizeof(gps_point_ll));
+                if (tmp == NULL)
+                    return;
+                if (no_time_head == NULL) {
+                    no_time_head = tmp;
+                    no_time_prev = tmp;
+                } else {
+                    no_time_prev->next = tmp;
+                    no_time_prev = tmp;
+                }
+                no_time_count += 1;
+                crt_point.time = no_time_count * 1000;
+                tmp->gp = crt_point;
+                tmp->next = NULL;
             } else
                 mlt_log_info(NULL,
                              "qxml_parse_tcx: skipping point due to time [%d] %f,%f - crt:%u.%u, "
@@ -1017,6 +1284,26 @@ void qxml_parse_tcx(QXmlStreamReader &reader, gps_point_ll **gps_list, int *coun
                              (unsigned) (last_time % 1000));
         }
     }
+    //if no points had time, we replace the real list with the no_time one
+    if (*count_pts == 0 && no_time_count > 0) {
+        mlt_log_info(NULL,
+                     "qxml_parse_gpx: all GPS points are missing time values, inserting each "
+                     "point at 1 second interval!");
+        *gps_list = no_time_head;
+        *count_pts = no_time_count;
+    } else if (*count_pts > 0 && no_time_count > 0) {
+        //cleanup the no_time list as we're not using it
+        mlt_log_info(NULL,
+                     "qxml_parse_gpx: %d GPS points have no time in original file and have been "
+                     "discarded (total valid points kept: %d)!",
+                     no_time_count,
+                     *count_pts);
+        while (no_time_head) {
+            gps_point_ll *tmp = no_time_head;
+            no_time_head = no_time_head->next;
+            free(tmp);
+        }
+    }
 }
 
 /* Reads file, parses it and stores the gps values in a gps_point_raw array (allocated inside) 
@@ -1050,9 +1337,9 @@ int qxml_parse_file(gps_private_data gdata)
             continue;
         }
         // mlt_log_info(NULL, " readNext element tag: %s, type=%d \n", qUtf8Printable(qxml_reader.name().toString()), qxml_reader.tokenType());
-        if (qxml_reader.name() == QString("TrainingCenterDatabase")) {
+        if (qxml_reader.name() == QStringLiteral("TrainingCenterDatabase")) {
             qxml_parse_tcx(qxml_reader, &gps_list_head, &count_pts);
-        } else if (qxml_reader.name() == QString("gpx")) {
+        } else if (qxml_reader.name() == QStringLiteral("gpx")) {
             qxml_parse_gpx(qxml_reader, &gps_list_head, &count_pts);
         } else {
             mlt_log_warning(
@@ -1073,6 +1360,13 @@ int qxml_parse_file(gps_private_data gdata)
 
     qxml_reader.clear(); //not sure if really needed if not reused
 
+    if (count_pts < 2) {
+        mlt_log_warning(gdata.filter,
+                        "qxml_parse_file: less than 2 gps points read (%d). Aborting. \n",
+                        count_pts);
+        return 0;
+    }
+
     *gdata.ptr_to_gps_points_r = (gps_point_raw *) calloc(count_pts, sizeof(gps_point_raw));
     gps_array = *gdata.ptr_to_gps_points_r; //just an alias
     if (gps_array == NULL) {
@@ -1119,11 +1413,24 @@ int qxml_parse_file(gps_private_data gdata)
         swap_to_180 = 0;
 
     // //debug result:
-    // 	for (int i = 0; i<*gdata.gps_points_size; i+=100) {
-    // 		gps_point_raw* crt_point = &gps_array[i];
-    // 		mlt_log_info(NULL, "_xml_parse_file read point[%d]: time:%d, lat:%f, lon:%f, ele:%f, distance:%f, hr:%f\n",
-    // 				i, crt_point->time/1000, crt_point->lat, crt_point->lon, crt_point->ele, crt_point->total_dist, crt_point->hr);
-    // 	}
+    // for (int i = 0; i < *gdata.gps_points_size; i += 100) {
+    //     gps_point_raw *crt_point = &gps_array[i];
+    //     mlt_log_info(NULL,
+    //                  "_xml_parse_file read point[%d]: time:%d, lat:%f, lon:%f, speed:%f, "
+    //                  "distance:%f, ele:%f, hr:%f, bearing:%f, cadence:%f, temp:%f, power:%f\n",
+    //                  i,
+    //                  crt_point->time / 1000,
+    //                  crt_point->lat,
+    //                  crt_point->lon,
+    //                  crt_point->speed,
+    //                  crt_point->total_dist,
+    //                  crt_point->ele,
+    //                  crt_point->hr,
+    //                  crt_point->bearing,
+    //                  crt_point->cad,
+    //                  crt_point->atemp,
+    //                  crt_point->power);
+    //}
 
     return 1;
-}
\ No newline at end of file
+}
