@@ -1,6 +1,6 @@
 /*
  * consumer_sdl.c -- A Simple DirectMedia Layer consumer
- * Copyright (C) 2017-2024 Meltytech, LLC
+ * Copyright (C) 2017-2025 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -216,6 +216,10 @@ int consumer_start(mlt_consumer parent)
         }
 
         // Initialize SDL video if needed.
+#ifdef __APPLE__
+        if (setup_sdl_video(self))
+            return 1;
+#else
         if (!SDL_WasInit(SDL_INIT_VIDEO)) {
             pthread_mutex_lock(&mlt_sdl_mutex);
             int ret = SDL_Init(SDL_INIT_VIDEO);
@@ -227,6 +231,7 @@ int consumer_start(mlt_consumer parent)
                 return 1;
             }
         }
+#endif
 
         pthread_create(&self->thread, NULL, consumer_thread, self);
     }
@@ -481,6 +486,20 @@ static int setup_sdl_video(consumer_sdl self)
     if (video_off || preview_off)
         return error;
 
+#ifdef __APPLE__
+    if (!SDL_WasInit(SDL_INIT_VIDEO)) {
+        pthread_mutex_lock(&mlt_sdl_mutex);
+        int ret = SDL_Init(SDL_INIT_VIDEO);
+        pthread_mutex_unlock(&mlt_sdl_mutex);
+        if (ret < 0) {
+            mlt_log_error(MLT_CONSUMER_SERVICE(&self->parent),
+                          "Failed to initialize SDL: %s\n",
+                          SDL_GetError());
+            return -1;
+        }
+    }
+#endif
+
 #ifdef MLT_IMAGE_FORMAT
     int image_format = mlt_properties_get_int(self->properties, "mlt_image_format");
 
