@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 from __future__ import print_function
-import mlt
+import mlt7 as mlt
 
 p = mlt.Properties()
 p.anim_set("foo", "bar", 10)
